////////////////////////////////////////////////////////////////////////
/*! \file   common_vertex.glsl
 *  \brief  Vertex shader common to all non-linear projections.
 *  \author J.D. Gascuel
 *  \sa sphere_fragment.glsl, parabol_fragment.glsl, lens_fragment.glsl, lambert_fragment.glsl
 */
////////////////////////////////////////////////////////////////////////
// $Id: common_vertex.glsl,v 1.13 2007/05/03 11:44:01 gascuel Exp $
// File History:
// [jDG 2006/12/05] : Creation from Cg version

////////////////////////////////////////////////////////////////////////

varying out vec3 iN;    // Surface normal, in camera coordinates.

#define NormalMatrixTranspose vec3(gl_ModelViewMatrixInverse[2])

// Shadow map data :
uniform mat4			uShadowMatrix;	// Projection from world to light frame.
varying out vec4        iS;				// Position in light's frame.

////////////////////////////////////////////////////////////////////////

void main()
{
    // Do not perform projection here : it is done in the geometry shader,
    // once all computation in world space are done :
    gl_Position = gl_ModelViewMatrix * gl_Vertex;

#ifdef SUBMODE_COLOR

#ifdef PER_PIXEL_LIGHTINGS
    iN = gl_NormalMatrix * gl_Normal;
    
#ifdef SHADOW_MAPPING
	iS =  uShadowMatrix * gl_Vertex;
#endif

#else	// no PER_PIXEL_LIGHTING
    // inline directionalLight(0) :
    vec3 N = normalize(gl_NormalMatrix * gl_Normal);
    vec3 V = normalize(gl_LightSource[0].position.xyz); // Light direction.
    vec3 H = gl_LightSource[0].halfVector.xyz;          // mid between V and Z.

    float nDotV = max(0.0, dot(N, V));
    float nDotH = max(0.0, dot(N, H));

    float pf = 0.0;
    if( nDotV > 0.0 && gl_FrontMaterial.shininess > 0.0 ) // Shinny only on the front side.
        pf = pow(nDotH, gl_FrontMaterial.shininess);

    // Just one light.
    gl_FrontColor = gl_BackColor 
    			  = gl_FrontLightModelProduct.sceneColor
                  + gl_FrontLightProduct[0].ambient
                  + gl_FrontLightProduct[0].diffuse * nDotV
                  + gl_FrontLightProduct[0].specular * pf
                  ;

    // Make sure light didn't change object's transparency.
    gl_FrontColor.a = gl_BackColor.a = gl_Color.a;
#endif	// PER_PIXEL_LIGHTING

#ifdef COLOR_TEXTURE
    gl_TexCoord[COLOR_TEXTURE] = gl_MultiTexCoord0;
#endif

#endif // SUBMODE_COLOR
}
